package xyz.lychee.lagfixer.libs.adventure.text.event;

import java.time.Duration;
import java.time.temporal.TemporalAmount;
import java.util.Objects;
import java.util.stream.Stream;
import xyz.lychee.lagfixer.libs.adventure.internal.Internals;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.examination.ExaminableProperty;

final class ClickCallbackOptionsImpl implements ClickCallback.Options {
   static final ClickCallback.Options DEFAULT = (new BuilderImpl()).build();
   private final int uses;
   private final Duration lifetime;

   ClickCallbackOptionsImpl(final int uses, final Duration lifetime) {
      this.uses = uses;
      this.lifetime = lifetime;
   }

   public int uses() {
      return this.uses;
   }

   public @NotNull Duration lifetime() {
      return this.lifetime;
   }

   public @NotNull Stream<? extends ExaminableProperty> examinableProperties() {
      return Stream.of(ExaminableProperty.of("uses", this.uses), ExaminableProperty.of("expiration", this.lifetime));
   }

   public String toString() {
      return Internals.toString(this);
   }

   static final class BuilderImpl implements ClickCallback.Options.Builder {
      private static final int DEFAULT_USES = 1;
      private int uses;
      private Duration lifetime;

      BuilderImpl() {
         this.uses = 1;
         this.lifetime = ClickCallback.DEFAULT_LIFETIME;
      }

      BuilderImpl(final ClickCallback.@NotNull Options existing) {
         this.uses = existing.uses();
         this.lifetime = existing.lifetime();
      }

      public ClickCallback.@NotNull Options build() {
         return new ClickCallbackOptionsImpl(this.uses, this.lifetime);
      }

      @NotNull
      public ClickCallback.Options.@NotNull Builder uses(final int uses) {
         this.uses = uses;
         return this;
      }

      @NotNull
      public ClickCallback.Options.@NotNull Builder lifetime(final @NotNull TemporalAmount lifetime) {
         this.lifetime = lifetime instanceof Duration ? (Duration)lifetime : Duration.from((TemporalAmount)Objects.requireNonNull(lifetime, "lifetime"));
         return this;
      }
   }
}
